==> classfile-0.4.0-sources.jar: org/glavo/classfile/impl/AbstractUnboundModel.java <==
/*
 * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.glavo.classfile.impl;

import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Stream;

import org.glavo.classfile.Attribute;
import org.glavo.classfile.AttributedElement;
import org.glavo.classfile.ClassfileElement;
import org.glavo.classfile.CompoundElement;

public abstract sealed class AbstractUnboundModel<E extends ClassfileElement>
        extends AbstractElement
        implements CompoundElement<E>, AttributedElement
        permits BufferedCodeBuilder.Model, BufferedFieldBuilder.Model, BufferedMethodBuilder.Model {
    private final List<E> elements;
    private List<Attribute<?>> attributes;

    public AbstractUnboundModel(List<E> elements) {
        this.elements = elements;
    }

    @Override
    public void forEachElement(Consumer<E> consumer) {
        elements.forEach(consumer);
    }

    @Override
    public Stream<E> elementStream() {
        return elements.stream();
    }

    @Override
    public List<E> elementList() {
        return elements;
    }

    @Override
    public List<Attribute<?>> attributes() {
        if (attributes == null)
            attributes = elements.stream()
                                 .filter(e -> e instanceof Attribute)
                                 .<Attribute<?>>map(e -> (Attribute<?>) e)
                                 .toList();
        return attributes;
    }
}

==> classfile-0.5.0-sources.jar: org/glavo/classfile/impl/AbstractUnboundModel.java <==
/*
 * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.glavo.classfile.impl;

import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Stream;

import org.glavo.classfile.Attribute;
import org.glavo.classfile.AttributedElement;
import org.glavo.classfile.ClassFileElement;
import org.glavo.classfile.CompoundElement;

public abstract sealed class AbstractUnboundModel<E extends ClassFileElement>
        extends AbstractElement
        implements CompoundElement<E>, AttributedElement
        permits BufferedCodeBuilder.Model, BufferedFieldBuilder.Model, BufferedMethodBuilder.Model {
    private final List<E> elements;
    private List<Attribute<?>> attributes;

    public AbstractUnboundModel(List<E> elements) {
        this.elements = elements;
    }

    @Override
    public void forEachElement(Consumer<E> consumer) {
        elements.forEach(consumer);
    }

    @Override
    public Stream<E> elementStream() {
        return elements.stream();
    }

    @Override
    public List<E> elementList() {
        return elements;
    }

    @Override
    public List<Attribute<?>> attributes() {
        if (attributes == null)
            attributes = elements.stream()
                                 .filter(e -> e instanceof Attribute)
                                 .<Attribute<?>>map(e -> (Attribute<?>) e)
                                 .toList();
        return attributes;
    }
}
